/*-
 * Copyright 2010 aian. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AIAN ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL aian OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of aian.
 */

/** Test class declaration and registration
 *
 * Global variables g_##name##_##group##_instance_ are const
 * object. So you cannot modify them, and need not operate them.
 * These instances are used for automatic registration of testing
 * object. Actual testing objects are constructed on heap memory.
 */
#ifndef ALPAKA__MACRO_SET_H
#define ALPAKA__MACRO_SET_H

#define TEST(group, name)                                                 \
  const class name##_##group##_ : public alpaka::Test {                   \
  public:                                                                 \
    name##_##group##_(const std::string& name, const std::string& group)  \
      : alpaka::Test(#name, #group) {                                     \
      if (!isRegistered_) {                                               \
        alpaka::TestRegistry::addTest(*this);                             \
        isRegistered_ = true;                                             \
      }                                                                   \
    };                                                                    \
    name##_##group##_(const name##_##group##_& other)                     \
      : alpaka::Test(other) {};                                           \
    virtual ~name##_##group##_() {};                                      \
  public:                                                                 \
    virtual void run(alpaka::ResultSet* result) throw ();                 \
    virtual alpaka::Test* clone() const throw () {                        \
      return new name##_##group##_(*this);                                \
    }                                                                     \
  private:                                                                \
    static bool isRegistered_;                                            \
  } g_##name##_##group##_instance_(#name, #group);                        \
  bool name##_##group##_::isRegistered_ = false;                          \
  void name##_##group##_::run(alpaka::ResultSet* result) throw ()

/** Checking conditions
 *
 */
#define CHECK_TRUE(expr)                                                  \
  if (!(expr)) {                                                          \
    result->add(alpaka::Failure(                                          \
      __FILE__, __LINE__,                                                 \
      #expr " that is expected to be true is actually false."));          \
  }
#define CHECK_FALSE(expr)                                                 \
  if ((expr)) {                                                           \
    result->add(alpaka::Failure(                                          \
      __FILE__, __LINE__,                                                 \
      #expr " that is expected to be false is actually true."));          \
  }
#define CHECK_OP_(lhs, op, rhs)                                           \
  if (!(lhs op rhs)) {                                                    \
    result->add(                                                          \
      alpaka::Failure(__FILE__, __LINE__, #lhs" "#op" "#rhs));            \
  }
#define CHECK_EQ(expected, actual)                                        \
  if (expected != actual) {                                               \
    result->add(alpaka::Failure(__FILE__, __LINE__, #expected, #actual)); \
  }
#define CHECK_NE(lhs, rhs) CHECK_OP_(lhs, !=, rhs)
#define CHECK_GE(lhs, rhs) CHECK_OP_(lhs, >=, rhs)
#define CHECK_LE(lhs, rhs) CHECK_OP_(lhs, <=, rhs)
#define CHECK_GT(lhs, rhs) CHECK_OP_(lhs,  >, rhs)
#define CHECK_LT(lhs, rhs) CHECK_OP_(lhs,  <, rhs)
#define CHECK_NOTHROW(expr)                                               \
  try {                                                                   \
    expr;                                                                 \
  } catch (...) {                                                         \
    result->add(alpaka::Failure(                                          \
      __FILE__, __LINE__, #expr " throws execption."));                   \
  }
#define CHECK_THROW(expr)                                                 \
  try {                                                                   \
    expr;                                                                 \
    result->add(alpaka::Failure(                                          \
      __FILE__, __LINE__, #expr " does not throws exception"));           \
  } catch (...) {                                                         \
  }

/** Assertion
 * By default, it just returns from their testing function.
 */
#ifndef ASSERTRTN_
#define ASSERTRTN_ return
#endif
#define ASSERTION(lhs, op, rhs)                                           \
  do {                                                                    \
    result->add(alpaka::Failure(__FILE__, __LINE__,                       \
      "ASSERT: " #lhs " " #op " " #rhs));                                 \
    ASSERTRTN_;                                                           \
  } while (0);
#define ASSERT_OP_(lhs, op, rhs)                                          \
  if (!(lhs op rhs)) {                                                    \
    ASSERTION(lhs, op, rhs);                                              \
  }
#define ASSERT_EQ(lhs, rhs) ASSERT_OP_(lhs, ==, rhs)
#define ASSERT_NE(lhs, rhs) ASSERT_OP_(lhs, !=, rhs)
#define ASSERT_GE(lhs, rhs) ASSERT_OP_(lhs, >=, rhs)
#define ASSERT_LE(lhs, rhs) ASSERT_OP_(lhs, <=, rhs)
#define ASSERT_GT(lhs, rhs) ASSERT_OP_(lhs,  >, rhs)
#define ASSERT_LT(lhs, rhs) ASSERT_OP_(lhs,  <, rhs)

#endif  // ALPAKA__MACRO_SET_H
