/*-
 * Copyright 2010 aian. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AIAN ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL aian OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of aian.
 */
#include <cassert>
#include <ostream>
#include <sstream>
#include "alpaka/Failure.h"

alpaka::Failure::Failure() throw () {
  assert(false);
}

alpaka::Failure::Failure(
  const std::string& file, int line, const std::string& cond) throw ()
  : mess_(cond), file_(file), line_(line) {
}

alpaka::Failure::Failure(
  const std::string& file, int line,
  const std::string& expected, const std::string& actual) throw ()
  : mess_(""), file_(file), line_(line) {
  std::stringstream ss;
  ss << "expected:" << expected << ", actual:" << actual;
  mess_ = ss.str();
}

alpaka::Failure::Failure(const Failure& other) throw ()
  : mess_(other.mess_), file_(other.file_), line_(other.line_) {
}

alpaka::Failure::~Failure() throw () {
}

alpaka::Failure&
alpaka::Failure::operator=(const Failure& rhs) throw () {
  if (this != &rhs) {
    mess_ = rhs.mess_;
    file_ = rhs.file_;
    line_ = rhs.line_;
  }
  return *this;
}

std::ostream&
alpaka::operator<<(std::ostream& lhs, const alpaka::Failure& rhs) {
  lhs << rhs.file_ << "(" << rhs.line_ << ") : " << rhs.mess_ << std::endl;
  return lhs;
}
