/*-
 * Copyright 2010 aian. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AIAN ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL aian OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of aian.
 */
#include <cassert>
#include <ostream>
#include "alpaka/Failure.h"
#include "alpaka/ResultSet.h"

alpaka::ResultSet::ResultSet() throw () {
}

alpaka::ResultSet::~ResultSet() throw () {
  try {
    clear();
  } catch (...) {
    assert(false);
  }
}

void
alpaka::ResultSet::add(const Failure& failure) throw () {
  results_.push_back(new Failure(failure));
}

int
alpaka::ResultSet::count() const throw () {
  return results_.size();
}

void
alpaka::ResultSet::clear() throw () {
  typedef std::vector<Failure*>::const_iterator Itr;
  for (Itr itr = results_.begin(); itr != results_.end(); ++itr) {
    delete *itr;
  }
  results_.clear();
}

std::ostream&
alpaka::operator<<(std::ostream& lhs, const alpaka::ResultSet& rhs) {
  typedef std::vector<alpaka::Failure*>::const_iterator Itr;
  for (Itr itr = rhs.results_.begin(); itr != rhs.results_.end(); ++itr) {
    lhs << **itr;
  }
  if (rhs.count() > 0) {
    lhs << rhs.count() << " tests failed.";
  } else {
    lhs << "Tests finished successfully.";
  }
  return lhs;
}
