/*-
 * Copyright 2010 aian. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AIAN ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL aian OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of aian.
 */
#include <cassert>
#include <iostream>
#include "alpaka/Test.h"
#include "alpaka/ResultSet.h"
#include "alpaka/TestRegistry.h"

alpaka::TestRegistry::TestRegistry() throw () {
}

alpaka::TestRegistry::~TestRegistry() throw () {
  try {
    clear();
  } catch (...) {
    assert(false);
  }
}

alpaka::TestRegistry&
alpaka::TestRegistry::instance() throw () {
  static TestRegistry me_;
  return me_;
}

void
alpaka::TestRegistry::addTest(const Test& test) throw () {
  instance().suite_.push_back(test.clone());
}

void
alpaka::TestRegistry::run() throw () {
  ResultSet result;
  typedef std::vector<Test*>::iterator Itr;
  for (Itr itr = suite_.begin(); itr != suite_.end(); ++itr) {
    std::cout << **itr << std::endl;
    (*itr)->init();
    (*itr)->run(&result);
    (*itr)->finalize();
    std::cout << result << std::endl;
    result.clear();
  }
}

void
alpaka::TestRegistry::clear() throw () {
  typedef std::vector<Test*>::iterator Itr;
  for (Itr itr = suite_.begin(); itr != suite_.end(); ++itr) {
    delete *itr;
  }
  suite_.clear();
}
